package com.cihub;

/**
 * CI/CD Hub Scaffold - Sample Application.
 *
 * <p>This class provides simple methods that demonstrate:
 * <ul>
 *   <li>JUnit 5 testable logic</li>
 *   <li>JaCoCo coverage instrumentation</li>
 *   <li>Checkstyle-compliant code style</li>
 *   <li>Mutation-testable operations (for PITest)</li>
 * </ul>
 *
 * <p>Replace this with your actual application code.
 */
public final class App {
    private App() {
    }

    /**
     * Multiply two integers.
     *
     * @param left the first operand
     * @param right the second operand
     * @return the product of left and right
     */
    public static int multiply(int left, int right) {
        return left * right;
    }
}
