# CI/CD Hub Scaffold - Python with setup.py
# Generated by: cihub scaffold --type python-setup
#
# This minimal project template is designed to pass all CI/CD Hub checks.
# Available tools (enable in .ci-hub.yml):
#   Testing:    pytest, hypothesis, mutmut
#   Linting:    ruff, black, isort, mypy
#   Security:   bandit, pip_audit, semgrep, trivy, codeql
#   Artifacts:  sbom, docker
#
# Next steps:
# 1. Run: cihub init --language python
# 2. Run: cihub smoke --full
# 3. Customize tools in .ci-hub.yml
#
# Note: For new projects, consider using pyproject.toml instead (python-pyproject).
# setup.py is supported for legacy compatibility.

from setuptools import find_packages, setup

setup(
    name="cihub_setup_sample",
    version="0.1.0",
    packages=find_packages(),
)
